/*******************************************************************************
 * 
 * jKiwi -- A virtual makeover and hairstyler application
 * Copyright (c) 2007-2008 Dan Mihai Ile, Maria Joao Roque Barbado Leal
 * 
 * This file is part of jKiwi.
 * 
 * jKiwi is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * jKiwi is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************/

package utils;

import java.lang.reflect.Method;

import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.PaletteData;
import org.eclipse.swt.internal.Converter;
import org.eclipse.swt.internal.gtk.OS;

public class GtkStockIconSWT extends GtkStockIcon {
	
	private static GtkStockIcon instance = null;
	
	public static GtkStockIcon getInstance() {
		if (instance == null) {
			instance = new GtkStockIconSWT();
		}
		return instance;
	}


	private Class<?> c = null;
	private Object o = null;
	private Method gtk_widget_get_default_style;
	private Method gtk_icon_set_render_icon;
	private Method gdk_pixbuf_get_width;
	private Method gdk_pixbuf_get_height;
	private Method gdk_pixbuf_get_rowstride;
	private Method gdk_pixbuf_get_has_alpha;
	private Method gdk_pixbuf_get_pixels;
	private Method g_object_unref;
	private Method gtk_icon_factory_lookup_default;
	
	@SuppressWarnings("static-access")
	private GtkStockIconSWT() { 
		try {
			c  = this.getClass().forName("org.eclipse.swt.internal.gtk.OS");
			o = c.newInstance();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		Method[] methods = c.getMethods();
		for (Method method : methods) {
			if (method.getName().equals("gtk_widget_get_default_style")) {
				gtk_widget_get_default_style = method;
			} else if (method.getName().equals("gtk_icon_set_render_icon")) {
				gtk_icon_set_render_icon = method;
			} else if (method.getName().equals("gdk_pixbuf_get_width")) {
				gdk_pixbuf_get_width = method;
			} else if (method.getName().equals("gdk_pixbuf_get_height")) {
				gdk_pixbuf_get_height = method;
			} else if (method.getName().equals("gdk_pixbuf_get_rowstride")) {
				gdk_pixbuf_get_rowstride = method;
			} else if (method.getName().equals("gdk_pixbuf_get_has_alpha")) {
				gdk_pixbuf_get_has_alpha = method;
			} else if (method.getName().equals("gdk_pixbuf_get_pixels")) {
				gdk_pixbuf_get_pixels = method;
			} else if (method.getName().equals("g_object_unref")) {
				g_object_unref = method;
			} else if (method.getName().equals("gtk_icon_factory_lookup_default")) {
				gtk_icon_factory_lookup_default = method;
			}
		}
	}
	
	public ImageData createImage(String name, int size, int orientation) {
		byte[] buffer = Converter.wcsToMbcs (null, name, true);
		int width = -1;
		int height = -1;
		int stride = -1;
		boolean hasAlpha = false;
		byte [] data = null;
		if (gtk_widget_get_default_style.getReturnType().getName().equals("int")) {
			try {
				int style = (Integer) gtk_widget_get_default_style.invoke(o);
				int pixbuf = (Integer) gtk_icon_set_render_icon.invoke(o, gtk_icon_factory_lookup_default.invoke(o, buffer), style,
						orientation, OS.GTK_STATE_NORMAL, size, 0, 0);
				if (pixbuf == 0) return null;
				width = (Integer) gdk_pixbuf_get_width.invoke(o, pixbuf);
				height = (Integer) gdk_pixbuf_get_height.invoke(o, pixbuf);
				stride = (Integer) gdk_pixbuf_get_rowstride.invoke(o, pixbuf);
				hasAlpha = (Boolean) gdk_pixbuf_get_has_alpha.invoke(o, pixbuf);
				int pixels = (Integer) gdk_pixbuf_get_pixels.invoke(o, pixbuf);
				
				data = new byte [stride * height];

				Class[] cTypes = new Class[] {byte[].class, int.class, int.class};
				c.getMethod("memmove", cTypes).invoke(o, data, pixels, data.length);
				
				g_object_unref.invoke(o, pixbuf);
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		} else {
			try {
				long style = (Integer) gtk_widget_get_default_style.invoke(o);
				long pixbuf = (Integer) gtk_icon_set_render_icon.invoke(o, gtk_icon_factory_lookup_default.invoke(o, buffer), style,
						orientation, OS.GTK_STATE_NORMAL, size, 0, 0);
				if (pixbuf == 0) return null;
				width = (Integer) gdk_pixbuf_get_width.invoke(o, pixbuf);
				height = (Integer) gdk_pixbuf_get_height.invoke(o, pixbuf);
				stride = (Integer) gdk_pixbuf_get_rowstride.invoke(o, pixbuf);
				hasAlpha = (Boolean) gdk_pixbuf_get_has_alpha.invoke(o, pixbuf);
				long pixels = (Integer) gdk_pixbuf_get_pixels.invoke(o, pixbuf);
				
				data = new byte [stride * height];
				
				Class[] cTypes = new Class[] {byte[].class, long.class, int.class};
				c.getMethod("memmove", cTypes).invoke(o, data, pixels, data.length);
				
				g_object_unref.invoke(o, pixbuf);
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		ImageData imageData = null;
		if (hasAlpha) {
			PaletteData palette = new PaletteData (0xFF000000, 0xFF0000, 0xFF00);
			imageData = new ImageData (width, height, 32, palette);
			byte [] alpha = new byte [stride * height];
			for (int y=0; y<height; y++) {
				for (int x=0; x<width; x++) {
					alpha [y*width+x] = data [y*stride+x*4+3];
					data [y*stride+x*4+3] = 0;
				}
			}
			imageData.setAlphas (0, 0, width * height, alpha, 0);
		} else {
			PaletteData palette = new PaletteData (0xFF0000, 0xFF00, 0xFF);
			imageData = new ImageData (width, height, 24, palette);
		}
		imageData.data = data;
		imageData.bytesPerLine = stride;
		return imageData;
	}

}
